/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <tizen.h>
#include <widget_app.h>
#include <widget_app_efl.h>
#include <dlog.h>

#include "main.h"
#include "view.h"

static void _widget_pressed_cb(void *user_data, Evas *evas, Evas_Object *obj, void *event_info);
static void _widget_released_cb(void *user_data, Evas *evas, Evas_Object *obj, void *event_info);

/**
 * @brief Sets image to given part.
 * @param[in] parent The object has part to which you want to set this image
 * @param[in] part_name The part name to which you want to set this image
 * @param[in] image_path The path of the image file
 */
void view_set_image(Evas_Object *parent, const char *part_name, const char *image_path)
{
	Evas_Object *image = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	image = elm_object_part_content_get(parent, part_name);
	if (image == NULL) {
		image = elm_image_add(parent);
		if (image == NULL) {
			dlog_print(DLOG_ERROR, LOG_TAG, "failed to create an image object.");
			return;
		}

		elm_object_part_content_set(parent, part_name, image);
	}

	if (EINA_FALSE == elm_image_file_set(image, image_path, NULL)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to set image.");
		return;
	}

	evas_object_show(image);

	return;
}

/**
 * @brief Creates a naviframe and sets to parent.
 * @param[in] parent The object to which you want to set naviframe object
 */
void view_create(widget_context_h context, int w, int h)
{
	widget_instance_data_s *wid = NULL;

	widget_app_context_get_tag(context, (void**) &wid);

	/* Create window */
	wid->win = view_create_widget_win(context, w, h);

	/* Create conformant */
	wid->conform = view_create_conformant_without_indicator(wid->win);
	if (wid->conform == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a conformant");
		return;
	}

	wid->root_width = w;
	wid->root_height = h;
}

/**
 * @brief Creates basic window for the widget with w, h size.
 * @param[in] context The context of widget instance
 * @param[in] w The pixel value for widget width
 * @param[in] h The pixel value for widget height
 */
Evas_Object *view_create_widget_win(widget_context_h context, int w, int h)
{
	Evas_Object *win = NULL;
	int ret;

	if (context == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get context.");
		return NULL;
	}

	/**
	 * Window
	 * Create and initialize elm_win.
	 * elm_win is mandatory to manipulate window.
	 */
	ret = widget_app_get_elm_win(context, &win);
	if (ret != WIDGET_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get window. err = %d", ret);
		return NULL;
	}

	evas_object_resize(win, w, h);
	evas_object_show(win);

	return win;
}

/**
 * @brief Creates a conformant without indicator for wearable app.
 * @param[in] win The object to which you want to set this conformant
 */
Evas_Object *view_create_conformant_without_indicator(Evas_Object *win)
{
	/**
	 * Conformant
	 * Create and initialize elm_conformant.
	 * elm_conformant is mandatory for base GUI to have proper size
	 * when indicator or virtual keypad is visible.
	 */
	Evas_Object *conform = NULL;

	if (win == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "window is NULL.");
		return NULL;
	}

	conform = elm_conformant_add(win);
	evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, conform);
	evas_object_show(conform);

	return conform;
}

/**
 * @brief Creates layout for alarm widget.
 * @param[in] context The context of widget instance
 * @param[in] file_path EDJ file path will be used for the layout
 */
void view_alarm_widget_create(widget_context_h context, const char *file_path)
{
	widget_instance_data_s *wid = NULL;

	/* Get Structure point from context */
	widget_app_context_get_tag(context, (void**) &wid);

	/* Create naviframe for management of several views easily */
	wid->nf = view_create_naviframe(wid->conform);
	if (wid->nf == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a naviframe.");
		evas_object_del(wid->win);
		return;
	}

	/**
	 * Eext circle
	 * Create Eext circle surface for circular naviframe.
	 * This make this app can show circular layout.
	 */
	wid->circle_surface = eext_circle_surface_naviframe_add(wid->nf);

	/* Create a layout for no alarm page.*/
	wid->layout = view_alarm_create_layout_no_alarmlist(wid->nf, file_path, "base_alarm", NULL);
	if (wid->layout == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a layout of no alarm.");
		evas_object_del(wid->win);
		return;
	}

	/* Push to naviframe */
	view_push_item_to_naviframe(wid->nf, wid->layout, NULL, NULL);
}

/**
 * @brief Creates layout for the first page of the alarm app when there is no alarm list.
 * @param[in] parent Object to which you want to add this layout
 * @param[in] file_path Path of EDJ will be used for this layout
 * @param[in] group_name Name of group in EDJ you want to set to
 * @param[in] user_data Data passed to the function
 */
Evas_Object *view_alarm_create_layout_no_alarmlist(Evas_Object *parent, const char *file_path, const char *group_name, void *user_data)
{
	Evas_Object *layout = NULL;

	/* Create layout with 'group_name' group in 'file_name' file */
	layout = view_create_layout(parent, file_path, group_name, NULL, NULL);
	if (layout == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create a layout of base alarm.");
		return NULL;
	}

	return layout;
}

/**
 * @brief Creates a layout to target parent object with EDJ file.
 * @param[in] parent The object to which you want to add this layout
 * @param[in] file_path File path of EDJ file will be used
 * @param[in] group_name The name of group in EDJ you want to set to
 * @param[in] cb_function The function will be called when back event is detected
 * @param[in] user_data The user data to be passed to the callback functions
 */
Evas_Object *view_create_layout(Evas_Object *parent, const char *file_path, const char *group_name, Eext_Event_Cb cb_function, void *user_data)
{
	Evas_Object *layout = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	/* Create layout by EDC(edje file) */
	layout = elm_layout_add(parent);
	elm_layout_file_set(layout, file_path, group_name);

	/* Layout size setting */
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	if (cb_function)
		eext_object_event_callback_add(layout, EEXT_CALLBACK_BACK, cb_function, user_data);

	evas_object_show(layout);

	return layout;
}

/**
 * @brief Makes popup with theme.
 * @param[in] parent The parent object
 * @param[in] timeout Duration that popup is showing
 * @param[in] text Text displayed on the popup
 */
Evas_Object *view_create_text_popup(Evas_Object *parent, double timeout, const char *text)
{
	Evas_Object *popup = NULL;
	Evas_Object *popup_layout = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	popup = elm_popup_add(parent);
	/* Set popup style as circle */
	elm_object_style_set(popup, "circle");
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	popup_layout = elm_layout_add(popup);
	/* Set layout theme of popup as a circle */
	elm_layout_theme_set(popup_layout, "layout", "popup", "content/circle");

	elm_object_content_set(popup, popup_layout);

	/* Set the time limit that make popup hidden */
	elm_popup_timeout_set(popup, timeout);

	if (text)
		view_set_text(popup_layout, "elm.text", text);

	evas_object_show(popup);

	return popup;
}

/**
 * @brief Sets the function will be called on the mouse events
 * @param[in] parent Layout will be received the event
 * @param[in] part_name Part name that the event is triggered
 * @param[in] event_type Event type
 * @param[in] event_cb The function will be called on the given event
 * @param[in] user_data The data will be passed to the event_cb
 */
void view_set_event_callback(Evas_Object *parent, const char *part_name, widget_event_type event_type, void *event_cb, void *user_data)
{
	Evas_Object *obj = NULL;

	obj = elm_object_part_content_get(parent, part_name);

	if (obj == NULL) {
		obj = (Evas_Object*) edje_object_part_object_get(elm_layout_edje_get(parent), part_name);
	}

	evas_object_data_set(obj, "callback_func_addr", (void*) event_cb);
	evas_object_data_set(obj, "widget_event_type", (void*) event_type);

	if (event_type == WIDGET_EVENT_PRESSED) {
		evas_object_event_callback_add(obj, EVAS_CALLBACK_MOUSE_DOWN, _widget_pressed_cb, user_data);
	} else if ((event_type == WIDGET_EVENT_RELEASED) || (event_type == WIDGET_EVENT_CLICKED)) {
		evas_object_event_callback_add(obj, EVAS_CALLBACK_MOUSE_UP, _widget_released_cb, user_data);
	}
}

/**
 * @brief Sets the Smart function will be called on the given event
 * @param[in] obj The object that the event is triggered
 * @param[in] event_type Event type
 * @param[in] event_cb The function will be called on the given event
 * @param[in] user_data The data will be passed to the event_cb
 */
void view_set_smart_callback(Evas_Object *obj, char *event_type, Evas_Smart_Cb event_cb, void *user_data)
{
	evas_object_smart_callback_add(obj, event_type, event_cb, user_data);
}

/**
 * @brief Sets text to the part.
 * @param[in] parent The object has part to which you want to set text
 * @param[in] part_name The part name to which you want to set the text
 * @param[in] text Text you want to set to the part
 */
void view_set_text(Evas_Object *parent, const char *part_name, const char *text)
{
	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	/* Set text of target part object */
	elm_object_part_text_set(parent, part_name, text);
}

/*
 * @brief Sends appointed signal and source to the EDJ object
 * @param[in] layout Object will receive the signal and source
 * @param[in] signal Appointed signal
 * @param[in] source Appointed source
 */
void view_send_signal_to_edje(Evas_Object *layout, const char *signal, const char *source)
{
	elm_object_signal_emit(layout, signal, source);
}

/**
 * @brief Creates a naviframe and sets to parent.
 * @param[in] parent The object to which you want to set naviframe object
 */
Evas_Object *view_create_naviframe(Evas_Object *parent)
{
	/**
	 * Naviframe
	 * Create and initialize elm_naviframe.
	 * Naviframe stands for navigation frame.
	 * elm_naviframe is a views manager for applications.
	 */
	Evas_Object *nf = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return NULL;
	}

	nf = elm_naviframe_add(parent);

	elm_object_part_content_set(parent, "elm.swallow.content", nf);
	eext_object_event_callback_add(nf, EEXT_CALLBACK_BACK, eext_naviframe_back_cb, NULL);
	eext_object_event_callback_add(nf, EEXT_CALLBACK_MORE, eext_naviframe_more_cb, NULL);

	evas_object_show(nf);

	return nf;
}

/**
 * @brief Pushes item to naviframe object.
 * @param[in] nf Naviframe that item will be added to
 * @param[in] item The object will be added to naviframe
 * @param[in] _pop_cb The function will be operated when this item is popped from naviframe
 * @param[in] cb_data The data passed to the '_pop_cb' function
 */
Elm_Object_Item* view_push_item_to_naviframe(Evas_Object *nf, Evas_Object *item, Elm_Naviframe_Item_Pop_Cb pop_cb, void *user_data)
{
	Elm_Object_Item* nf_it = NULL;

	if (nf == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "naviframe is NULL.");
		return NULL;
	}

	if (item == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "item is NULL.");
		return NULL;
	}

	nf_it = elm_naviframe_item_push(nf, NULL, NULL, NULL, item, "empty");

	if (pop_cb != NULL)
		elm_naviframe_item_pop_cb_set(nf_it, pop_cb, user_data);

	return nf_it;
}

/**
 * @brief Creates and sets button.
 * @param[in] parent Object to which you want to set the button
 * @param[in] part_name Name of part to which you want to set the button
 * @param[in] style Style of the button
 * @param[in] image_path Path of image file will be used as button icon
 * @param[in] text The text will be written on the button
 * @param[in] down_cb Function will be operated when the button is pressed
 * @param[in] up_cb Function will be operated when the button is released
 * @param[in] clicked_cb Function will be operated when the button is clicked
 * @param[in] user_data Data passed to the 'clicked_cb' function
 */
void view_set_button(Evas_Object *parent, const char *part_name, const char *style, const char *image_path, const char *text,
		Evas_Object_Event_Cb down_cb, Evas_Object_Event_Cb up_cb, Evas_Smart_Cb clicked_cb, void *user_data)
{
	Evas_Object *btn = NULL;

	if (parent == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "parent is NULL.");
		return;
	}

	btn = elm_button_add(parent);
	if (btn == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to create button.");
		return;
	}

	if (style)
		elm_object_style_set(btn, style);

	evas_object_size_hint_weight_set(btn, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_object_part_content_set(parent, part_name, btn);

	if (text)
		elm_object_text_set(btn, text);

	if (image_path)
		view_set_image(btn, NULL, image_path);

	if (down_cb)
		evas_object_event_callback_add(btn , EVAS_CALLBACK_MOUSE_DOWN, down_cb, user_data);
	if (up_cb)
		evas_object_event_callback_add(btn, EVAS_CALLBACK_MOUSE_UP, up_cb, user_data);
	if (clicked_cb)
		evas_object_smart_callback_add(btn, "clicked", clicked_cb, user_data);

	evas_object_show(btn);
}

/**
 * @brief Destroys window and frees its resources.
 */
void view_widget_destroy(void *user_data)
{
	widget_instance_data_s *wid = NULL;

	wid = user_data;
	if (wid == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to destroy data.");
		return;
	}

	evas_object_del(wid->win);

	free(wid->alarm_id);
	free(wid);
}

/**
 * @brief Resizes the window.
 * @param[in] user_data Structure has informations for managing this widget
 * @param[in] w The pixel value for widget width
 * @param[in] h The pixel value for widget height
 */
void view_resize(void *user_data, int w, int h)
{
	widget_instance_data_s *wid = NULL;

	wid = user_data;
	if (wid == NULL) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to widget data.");
		return;
	}

	evas_object_resize(wid->win, w, h);
}

/**
 * @brief The function will be called when widget is pressed.
 * @param[in] user_data Structure has informations for managing this widget
 * @param[in] evas Evas object
 * @param[in] obj Object that the event is triggered
 * @param[in] event_info The information regarding with the event
 */
static void _widget_pressed_cb(void *user_data, Evas *evas, Evas_Object *obj, void *event_info)
{
	widget_instance_data_s *wid = user_data;
	if (wid == NULL) {
		return;
	}

	widget_event_cb func = evas_object_data_get(obj, "callback_func_addr");

	func(wid);
}

/**
 * @brief The function will be called when widget is released.
 * @param[in] user_data Structure has informations for managing this widget
 * @param[in] evas Evas object
 * @param[in] obj Object that the event is triggered
 * @param[in] event_info The information regarding with the event
 */
static void _widget_released_cb(void *user_data, Evas *evas, Evas_Object *obj, void *event_info)
{
	widget_instance_data_s *wid = user_data;
	Evas_Event_Mouse_Up *ev = event_info;
	Evas_Coord x, y, w, h;

	if (wid == NULL) {
		return;
	}

	widget_event_cb func = (widget_event_cb) evas_object_data_get(obj, "callback_func_addr");
	widget_event_type type = (widget_event_type) evas_object_data_get(obj, "widget_event_type");

	if (type == WIDGET_EVENT_RELEASED) {
		if (func) {
			func(wid);
		}
	} else if (type == WIDGET_EVENT_CLICKED) {
		evas_object_geometry_get(obj, &x, &y, &w, &h);

		if (!ELM_RECTS_POINT_OUT(x, y, w, h, ev->canvas.x, ev->canvas.y)) {
			if (func) {
				func(wid);
			}
		}
	}
}
/* End of the file */
